<?php
/*
Plugin Name: How To Do it Right 1: URL Rewriting with WordPress
Description: How to do it Right 1: URL Rewriting with WordPress
Author: Dion Hulse
Version: 1.0
Author URI: http://dd32.id.au/
*/

$htdir_url_rewriting = new How_to_do_it_right_URL_Rewriting();
class How_to_do_it_right_URL_Rewriting {
	
	var $is_product = false;
	
	function How_to_do_it_right_URL_Rewriting() {
		// Never run any code other than add_action/add_filter when your plugin is included. We'll hook to init and run everything then.
		add_action('init', array(&$this, 'init'));
		// Note however, That this init hook will NOT be called when the plugin is activated, for that, we need to call register_activation_hook() on include:
		register_activation_hook( __FILE__, array(&$this, 'activation') );
		
		if ( ! defined('SAVEQUERIES') ) // This is for debug purposes, It saves all the queries performed so we can check it later on in the template.
			define('SAVEQUERIES', true);
	}
	function activation() {
		$this->init(); // As stated above, we're after init, so initialise the plugin now.
		global $wp_rewrite;
		$wp_rewrite->flush_rules(false); // false => hard flush will re-write .htaccess, Theres no need for that right now.
	}
	function init() {
		global $wp_rewrite;
		// WordPress has been loaded, Time to do some magic.
		
		// 1. Plugin Initialisation
		// First of all, Lets register our post_type, and allow it to appear in search's
		if ( function_exists('register_post_type') ) // WordPress 2.9+, But not required for functionality in 2.8
			register_post_type( 'product', array('exclude_from_search' => false) );
		
		// 2. Rewrite Initialisation
		// Next up we need to add some rewrite rules to allow easy access to products.
		add_rewrite_tag('%product%', '([^/]+)');
		$wp_rewrite->add_permastruct('products', 'products/%product%/');

		// 3. Query Initialisation
		// Next, Lets setup WP_Query to handle our requests:
		// This is where we hook into WordPress and tell the Query parser how to handle our requests properly.
		add_filter('parse_query', array(&$this, '_query'));

		// 4. Template Modifications
		// Finally, We're going to modify a few Template functions to do what we want.
		
		// Lets correct the Products permalink to the correct URL structure we've created.
		add_filter('post_link', array(&$this, '_product_permalink'), 20, 2);
		// Lets use a custom template for our product display:
		add_filter('single_template', array(&$this, '_product_template') );
		
		//Final, ignore this part, its just the admin stuff:
		add_action('admin_menu', array(&$this, 'admin_menu') );
	}
	
	function _query($request) {
		// First, We check to see if this request is for us or not.
		if ( empty($request->query_vars['product']) )
			return $request;

		// Ok, a Product is specified
		$this->is_product = true;

		// We want to query for a Product by name
		$request->query_vars['name'] = $request->query_vars['product'];
		// And query for post of post_Type product.
		$request->query_vars['post_type'] = 'product';

		// This is the only hacky piece. We need to tell WordPress that this is a single page
		$request->is_single = true;
		$request->is_home = false;

		return $request;
	}
	
	function _product_permalink($link, $post) {
		if ( 'product' != $post->post_type )
			return $link;

		// Lets create our link now:
		global $wp_rewrite;
		$pagestruct = $wp_rewrite->get_page_permastruct();

		// We base this off the Page structure, as this allows us to create a proper link, Even in the case that we're using index.php/<slug> links.
		// This will create links such as:
		//  <HOME>/prodcuts/<PRODUCT SLUG>/
		//  <HOME>/index.php/prodcuts/<PRODUCT SLUG>/
		//  <HOME>/?product=<PRODUCT SLUG>
		if ( !empty($pagestruct) ) {
			$link = trailingslashit(get_bloginfo('home')) . str_replace('%pagename%', 'products/' . $post->post_name, $pagestruct);
			$link = user_trailingslashit($link);
		} else {
			$link = trailingslashit(get_bloginfo('home')) . '?product=' . $post->post_name;
		}
		return $link;
	}
	function _product_template($template) {
		// First up, Lets not mess with other object types
		if ( ! $this->is_product )
			return $template;
		// Eg: Custom template bundled with the plugin:
		return dirname(__FILE__) . '/templates/product.php';
	}
	
	//----------------------------------------------------------------
	// Admin stuff unrelated to the purpose of this plugin:
	function admin_menu() {
		include dirname(__FILE__) . '/HTDIR_URL_Rewrite_admin.php';
		$this->admin = new How_to_do_it_right_URL_Rewriting_Admin(&$this);
	}

}